#include "Panel_CP2K.h"

#include "Log.h"

#include "Defs.h"
#include "Path.h"
#include "User.h"
#include "Model.h"
#include "Model_kit.h"
#include "Atom.h"
#include "Main_window.h"

#include <QCloseEvent>
#include <QDir>

#include <sstream>
#include <fstream>
#include <math.h>

namespace MM
{
using namespace std;

Panel_CP2K::
Panel_CP2K ()
:
    Panel ("CP2K", "left"),

    model_                  (0),
    process_                (new QProcess(this)),
    output_                 (setup_CP2K_dir() + "CP2K-output"),
    //output_                 (Path::report() + "CP2K-output"),
    player_                 (Main_window::singleton()),
    read_atom_coords_flag_  (false),
    read_charges_flag_      (false),
    read_moments_flag_      (false),
    user_stop_flag_         (false),
    success_                (false),
    optimization_it_        (0)
{
    setupUi (this);

    //====================================================

    //listWidget_Basis->item(5)->setHidden (true);

    //====================================================


    connect (comboBox_RUNTYP,   SIGNAL(currentIndexChanged (const QString &)),
             this,              SLOT  (show_options        (const QString &)));

    process_->setReadChannel (QProcess::StandardOutput);
    process_->setWorkingDirectory (CP2K_dir());

    connect (process_,          SIGNAL(started()),
             this,              SLOT  (started()));

    connect (process_,          SIGNAL(finished (int, QProcess::ExitStatus)),
             this,              SLOT  (finished (int, QProcess::ExitStatus)));

    connect (process_,          SIGNAL(readyRead()),
             this,              SLOT  (read()));

    listWidget_Basis->setCurrentRow (7);
}

Panel_CP2K::
~Panel_CP2K ()
{
    if (process_->isOpen ())
        process_->kill ();

    if (output_.isOpen ())
    {
        output_.flush ();
        output_.close ();
    }
}

void Panel_CP2K::
turn (bool on)
{
    //Main_window *player = Main_window::singleton();

    //if (process_->state() == QProcess::NotRunning)
    if (!player_->player_on_)
    {
        player_->toolButton_Stop   -> disconnect ();
        player_->toolButton_Back   -> disconnect ();
        player_->toolButton_Tune   -> disconnect ();
        player_->toolButton_Run    -> disconnect ();
        player_->toolButton_Start  -> disconnect ();
        player_->toolButton_Forward-> disconnect ();
        player_->toolButton_Pause  -> disconnect ();

        if (on)
        {
            player_->tabWidget_bottom->setCurrentWidget (player_->tab_Run);

            connect (player_->toolButton_Stop,   SIGNAL(clicked()), this,
                                 SLOT  (stop   ()));
            connect (player_->toolButton_Tune,   SIGNAL(clicked()), this,
                                 SLOT  (tune   ()));
            connect (player_->toolButton_Run,    SIGNAL(clicked()), this,
                                 SLOT  (run    ()));
            connect (player_->toolButton_Start,  SIGNAL(clicked()), this,
                                 SLOT  (start  ()));
            set_stop_state ();
        }
        else 
        {
            player_->toolButton_Stop   -> setEnabled (false);
            player_->toolButton_Back   -> setEnabled (false);
            player_->toolButton_Tune   -> setEnabled (false);
            player_->toolButton_Run    -> setEnabled (false);
            player_->toolButton_Start  -> setEnabled (false);
            player_->toolButton_Forward-> setEnabled (false);
            player_->toolButton_Pause  -> setEnabled (false);
        }
    }
}

void Panel_CP2K::
closeEvent (QCloseEvent * event)
{
    if (process_->state() == QProcess::Running)
        event->ignore ();
}

Text const & Panel_CP2K::
setup_CP2K_dir ()
{
    CP2K_dir_ = Path::CP2K();
    //ifstream path_file ((CP2K_dir_ + "path.txt").c_str());

    //if (path_file)
    //{
    //    string new_path;
    //    getline (path_file, new_path);
    //    CP2K_dir_ = new_path.c_str();
    //}

    return CP2K_dir_;
}

void Panel_CP2K::
show_options (const QString & text)
{
    for (int i=0;  i<stackedWidget->count();  ++i)
    {
        if (stackedWidget->widget(i)->objectName() == text)
            stackedWidget->setCurrentIndex (i);
    }
}

void Panel_CP2K::
run ()
{
    if (process_->state() == QProcess::NotRunning)
    {
        tune  ();
        start ();
    }
    else
        stop();
}

void Panel_CP2K::
tune ()
{
    model_ = &model();

    create_input ();
    to_user().status ("CP2K input generated.");
}

void Panel_CP2K::
start ()
{
    model_ = &model();

    if (process_->state() == QProcess::NotRunning)
    {
        to_user().info ("\n___________________________________________");

        if (output_.isOpen ())
            output_.close ();

        if (!output_.open (QIODevice::WriteOnly/* | QIODevice::Text*/))
        {
            to_user().error ("Can not open 'output' file.");     
            return;
        }

        QString name = stackedWidget->currentWidget()->objectName();//fix to state
        if (name == "OPTIMIZE")
        {
            optimization_it_ = 1;
        }
        else
            optimization_it_ = 0;

        QDir dir(Path::report());
        dir.remove ("CP2K-1_0.resp");
        dir.remove ("CP2K-moments-1_0.dat");

        Text        program = CP2K_dir() +"cp2k";
        QStringList arguments;
        arguments << "-i" << "CP2K.input";

        user_stop_flag_ = false;

        process_->start (program, arguments);

        bool started = process_->waitForStarted (3000);
        if (!started)
        {
            to_user().fatal_error (
                "Can not start CP2K.\n\n"
                "Please check that there are \"cp2k\" and dynamic library "
                "files in Ascalaph/CP2K folder.\n"
                );
            set_stop_state ();
        }
    }
    else
        stop();
}

void Panel_CP2K::
stop ()
{
//    pushButton_Check->setEnabled (false);
    user_stop_flag_ = true;
    player_->player_on_ = false;
    process_->kill();

    to_user ().info ("Simulation was stopped by the user.");
}

void Panel_CP2K::
started ()
{
    to_user ().status ("CP2K started");
    set_run_state ();
}

void Panel_CP2K::
finished (int, QProcess::ExitStatus exitStatus)
{
    //qApp->processEvents(QEventLoop::ExcludeSocketNotifiers);

    if (exitStatus == QProcess::CrashExit && !user_stop_flag_)
    {
        Text message ("CP2K crashed. ");
        to_user ().status (message);

        switch (process_->error ())
        {
        case QProcess::FailedToStart:
            to_user ().error (message +
                "The process failed to start. "
                "Either the invoked program is missing, or you may have "
                "insufficient permissions to invoke the program.");
            break;

        case QProcess::Crashed:
            to_user ().error (message +
                "The process crashed some time after "
                "starting successfully.");
            break;

        case QProcess::Timedout:
            to_user ().error (message + "Timed out");
            break;

        case QProcess::WriteError:
            to_user ().error (message +
                "An error occurred when attempting to write to the process.");
            break;

        case QProcess::ReadError:
            to_user ().error (message +
                "An error occurred when attempting to read from "
                "the process. For example, the process may not be running.");
            break;

        case QProcess::UnknownError:
            to_user ().error (message + "An unknown error occurred.");
            break;
        }
    }
    else
    {
        //read ();
        output_.flush ();

        Text message ("CP2K finished ");
        if (success_)
            message += "NORMALLY";
        //else
        //    to_user ().warning ("Execution of CP2K terminated ABNORMALLY.");

        to_user ().status (message);
    }

    read ();
    set_stop_state ();
}

void Panel_CP2K::
set_stop_state ()
{
    player_->player_on_ = false;

    player_->toolButton_Stop   -> setEnabled (false);
    player_->toolButton_Tune   -> setEnabled (true);
    player_->toolButton_Run    -> setEnabled (true);
    player_->toolButton_Start  -> setEnabled (true);

    if (this->isHidden ())
        turn (false);
}

void Panel_CP2K::
set_run_state ()
{
    player_->player_on_ = true;

    player_->toolButton_Stop   -> setEnabled (true);
    player_->toolButton_Tune   -> setEnabled (false);
    player_->toolButton_Run    -> setEnabled (false);
    player_->toolButton_Start  -> setEnabled (false);
}

void Panel_CP2K::
read ()
{
    char    buf[1024];
    //Text    text;
    char *  result = 0;
    string  stub;
    //Text    message;

    while (process_->canReadLine ())
    {
        qint64 bytes = process_->readLine (buf, sizeof(buf));
        output_.write (buf, bytes);

        for (int i=0;  i<1024;  ++i)
        {
            if (buf[i] == '\0')
            {
                buf[i-2] = 0x0A;
                buf[i-2] = '\0';
            }
        }

        if (read_atom_coords_flag_)
        {
            read_atom_coords (buf);
            continue;
        }

        if (read_charges_flag_)
        {
            read_charges (buf);
            continue;
        }

        if (read_moments_flag_)
        {
            read_moments (buf);
            continue;
        }

        //=====================================================================
        qApp->processEvents (QEventLoop::ExcludeSocketNotifiers);

        //result = strstr (buf, "COORDINATES OF ALL ATOMS ARE (ANGS)");
        //if (result || 
        //    strstr (buf, "COORDINATES OF SYMMETRY UNIQUE ATOMS (ANGS)"))
        //{
        //    read_atom_coords_flag_ = true;
        //    continue;
        //}

        result = strstr (buf, "RESP charges");
        if (result)
        {
            to_user().info ("");
            read_charges_flag_ = true;
            read_charges (buf);
            continue;
        }

        result = strstr (buf, "Dipole moment");
        if (result)
        {
            to_user().info ("");
            to_user().info ("Dipole moment [Debye]");
            read_moments_flag_ = true;
            continue;
        }

        double e;

        result = strstr (buf, "Total energy:");
        if (result)
        {
            if (optimization_it_!=0)
                continue;

            istringstream in (buf);
            //int    it;
            in >> stub >> stub >> e;

            Text message;
            message += Text (" E        = ");
            message += Text (e, 15);
            message += " Hartree   ";
            message += Text (e*Constant::Hartree_to_kcal, 15);
            message += " kcal/mol";
            to_user ().info (message);
            message.clear ();

            continue;
        }

        //if (message.is_not_empty())
        //{
        //    to_user ().info (message);
        //    message.clear ();
        //}

        //=====================================================================
        char *abnormal;

        abnormal= 
            strstr (buf, "EXECUTION OF CP2K TERMINATED ABNORMALLY");
        if (abnormal)
            to_user ().fatal_error (buf);

        abnormal = strstr (buf, "*** ERROR");
        if (abnormal || strstr (buf, "ERROR!"))
            to_user ().error (buf);

        char *normal = strstr (buf, "EXECUTION OF CP2K TERMINATED NORMALLY");
        if (normal)
            success_ = true;
    }
    //log () << text;
    output_.flush ();
//    qApp->processEvents (QEventLoop::ExcludeSocketNotifiers);
}

void Panel_CP2K::
read_atom_coords (char *buf)
{
    static int n = -2;
    string stub;
    double x, y, z;

    int length = strlen (buf);
    if (length < 4)
    {
        read_atom_coords_flag_ = false;
        n = -2;
        model_->kit().event().conformation_changed ();
        return;
    }

    if (n >=0)
    {
        Atom & atom = model_->atom (n);

        istringstream in (buf);
        in >> stub >> stub >> x >> y >> z;
        atom.set_x (x);
        atom.set_y (y);
        atom.set_z (z);
    }

    ++n;
}

void Panel_CP2K::
read_charges (char *buf)
{
    static int n = -3;
    string stub;
    double charge;

    if (n >= 0 && buf[2] == 'T')
    {
        read_charges_flag_ = false;
        n = -3;
        return;
    }

    if (n >=0)
    {
        Atom & atom = model_->atom (n);

        istringstream in (buf);
        in >> stub >> stub >> charge;
        atom.set_charge (charge);

        to_user().info (buf);
    }

    ++n;
}

void Panel_CP2K::
read_moments (char *buf)
{
    int length = strlen (buf);
    if (length <= 2)
    {
        read_moments_flag_ = false;
        return;
    }

    to_user().info (buf);
}

void Panel_CP2K::
create_input ()
{
    success_ = false;
    QString job = stackedWidget->currentWidget()->objectName();
    
    QString XC_FUNCTIONAL = comboBox_DFT->currentText();
    if (XC_FUNCTIONAL == "PADE (LDA)")
        XC_FUNCTIONAL = "PADE";

    Text text;
    ofstream out ((CP2K_dir() + "CP2K.input").c_str());

    out << "&FORCE_EVAL                             " << endl;
    out << "  METHOD Quickstep                      " << endl;
    out << "  &DFT                                  " << endl;
    out << "  CHARGE " << spinBox_Charge->value()     << endl;
    out << "    BASIS_SET_FILE_NAME GTH_BASIS_SETS  " << endl;
    out << "    POTENTIAL_FILE_NAME GTH_POTENTIALS  " << endl;
    out << endl;

    out << "    &MGRID                                  " << endl;
    out << "      CUTOFF " << doubleSpinBox_cutof->value() 
        << "            ! Default 280   " << endl;
    out << "    &END MGRID                              " << endl;
    out << endl;

    out << "    &QS                                             " << endl;
    out << "      METHOD GPW                                    " << endl;
    out << "      EXTRAPOLATION ASPC 	! Default               " << endl;
    out << "    &END QS                                         " << endl;
    out << "                                                    " << endl;
    out << "    &SCF                                            " << endl;
    out << "      MAX_SCF " << (comboBox_Calc->currentText() == "Accurate" ? 10000 : 1000)
        << "                                                    " << endl;
    out << "      EPS_SCF 1.0E-" 
        << (comboBox_Calc->currentText() == "Accurate" ? 6 : 5)
        << "  		! Default 1.0E-05                           " << endl;
    out << "      SCF_GUESS ATOMIC  	! Default               " << endl;
    out << "      &OT                                           " << endl;
    out << "        ENERGY_GAP " 
        << (comboBox_Calc->currentText() == "Accurate" ? 0.001 : 0.2)
        << "  	! Default 0.2 to 0.001 with a FULL_ALL.         " << endl;
    out << "        PRECONDITIONER FULL_ALL	!               for almost all systems" << endl;
    out << "        MINIMIZER  DIIS  	!               50% faster                " << endl;
    out << "!        MINIMIZER  CG  	! Default 	for difficult systems         " << endl;
    out << "      &END                                          " << endl;
    out << "    &END SCF                                        " << endl;
    out << endl;

    out << "    &XC                                             " << endl;
    out << "      &XC_FUNCTIONAL " << Text(XC_FUNCTIONAL).c_str() 
        << " ! B3LYP BLYP BP HCTH120 NONE NO_SHORTCUT OLYP PADE PBE PBE0 TPSS" << endl;
    out << "      &END XC_FUNCTIONAL                            " << endl;
    out << "    &END XC                                         " << endl;
    out << "                                                    " << endl;
    out << "    &PRINT                                          " << endl;
    out << "      &MOMENTS                                      " << endl;
    out << " !       FILENAME  ../Report/CP2K                    " << endl;
    out << "        MAX_MOMENT 1                                " << endl;
    out << "        PERIODIC F                                  " << endl;
    out << "      &END MOMENTS                                  " << endl;
    out << "    &END                                            " << endl;
    out << "                                                    " << endl;
    out << "    &POISSON                                        " << endl;
    out << "      PERIODIC NONE                                 " << endl;
    out << "      POISSON_SOLVER  WAVELET  		! Default PERIODIC    MT  WAVELET" << endl;
    out << "    &END                                            " << endl;
    out << "                                                    " << endl;
    out << "  &END DFT                                          " << endl;
    out << endl;

    double cell = doubleSpinBox_cell->value();
    out << "  &SUBSYS                                           " << endl;
    out << "    &CELL       ! 19 MGRID cutof 180, 18 - 220, 16 - 180, 15 - 320" << endl;
    out << "      ABC " << cell << " " << cell << " " << cell     << endl;
    out << "      PERIODIC NONE                                 " << endl;
    out << "    &END CELL                                       " << endl;
    out << "                                                    " << endl;
    out << "    &TOPOLOGY                                       " << endl;
    out << "      CENTER_COORDINATES                            " << endl;
    out << "    &END                                            " << endl;
    out << "                                                    " << endl;
    out << "    &COORD                                          " << endl;
    out << data().c_str() << endl;
    out << "    &END COORD                                      " << endl;
    out << "                                                    " << endl;
    out << basis().c_str() << endl;
    out << "  &END SUBSYS                                       " << endl;
    out << "                                                    " << endl;

    if (job == "Electrostatics")
    {
    out << "    &PROPERTIES                                     " << endl;
    out << "      &RESP                                         " << endl;
    out << "        &PROGRAM_RUN_INFO                           " << endl;
    out << "  !        FILENAME  ../Report/CP2K                  " << endl;
    out << "        &END                                        " << endl;
    out << "                                                    " << endl;
    out << "        RMIN " << doubleSpinBox_RMIN->value() 
        << "                                                    " << endl;
    out << "        RMAX " << doubleSpinBox_RMAX->value()  
        << "                                                    " << endl;    
    out << "        STRIDE " << spinBox_STRIDE->value() 
        << "                                    " << endl;
    out << "        RESTRAIN_HEAVIES_TO_ZERO F                  " << endl;    
    out << "                                                    " << endl;
    out << "      &END                                          " << endl;
    out << "      &FIT_CHARGE                                   " << endl;
    out << "      &END                                          " << endl;
    out << "    &END                                            " << endl;
    out << "                                                    " << endl;
    }

    out << "&END FORCE_EVAL                                     " << endl;
    out << "                                                    " << endl;
    out << "&GLOBAL                                             " << endl;
    out << "  PROJECT ~tmp                                      " << endl;
    out << "  RUN_TYPE ENERGY                                   " << endl;
    out << "  PRINT_LEVEL LOW                                   " << endl;
    out << "  SAVE_MEM                                          " << endl;
    out << "&END GLOBAL                                         " << endl;
    out << endl;
}

Text Panel_CP2K::
basis ()
{
    Text text;

    if (comboBox_DFT->currentText() == "BLYP")
    {
        Text basis (listWidget_Basis->currentItem()->text());
        text += 
        "    &KIND H \n"
        "      BASIS_SET ";
        text += basis;
        text += " \n";
        text += 
        "      POTENTIAL GTH-BLYP-q1 \n"
        "    &END KIND \n"
        "    &KIND C \n"
        "      BASIS_SET ";
        text += basis;
        text += " \n";
        text += 
        "      POTENTIAL GTH-BLYP-q4 \n"
        "    &END KIND \n"
        "    &KIND N \n"
        "      BASIS_SET ";
        text += basis;
        text += " \n";
        text += 
        "      POTENTIAL GTH-BLYP-q5 \n"
        "    &END KIND \n"
        "    &KIND O \n"
        "      BASIS_SET ";
        text += basis;
        text += " \n";
        text += 
        "      POTENTIAL GTH-BLYP-q6 \n"
        "    &END KIND \n";
    }
    else if (comboBox_DFT->currentText() == "PADE (LDA)")
    {
        Text basis (listWidget_Basis->currentItem()->text());
        text += 
        "    &KIND H \n"
        "      BASIS_SET ";
        text += basis;
        text += " \n";
        text += 
        "      POTENTIAL GTH-PADE-q1 \n"
        "    &END KIND \n"
        "    &KIND C \n"
        "      BASIS_SET ";
        text += basis;
        text += " \n";
        text += 
        "      POTENTIAL GTH-PADE-q4 \n"
        "    &END KIND \n"
        "    &KIND N \n"
        "      BASIS_SET ";
        text += basis;
        text += " \n";
        text += 
        "      POTENTIAL GTH-PADE-q5 \n"
        "    &END KIND \n"
        "    &KIND O \n"
        "      BASIS_SET ";
        text += basis;
        text += " \n";
        text += 
        "      POTENTIAL GTH-PADE-q6 \n"
        "    &END KIND \n";
    }
    else if (comboBox_DFT->currentText() == "PBE")
    {
        Text basis (listWidget_Basis->currentItem()->text());
        text += 
        "    &KIND H \n"
        "      BASIS_SET ";
        text += basis;
        text += " \n";
        text += 
        "      POTENTIAL GTH-PBE-q1 \n"
        "    &END KIND \n"
        "    &KIND C \n"
        "      BASIS_SET ";
        text += basis;
        text += " \n";
        text += 
        "      POTENTIAL GTH-PBE-q4 \n"
        "    &END KIND \n"
        "    &KIND N \n"
        "      BASIS_SET ";
        text += basis;
        text += " \n";
        text += 
        "      POTENTIAL GTH-PBE-q5 \n"
        "    &END KIND \n"
        "    &KIND O \n"
        "      BASIS_SET ";
        text += basis;
        text += " \n";
        text += 
        "      POTENTIAL GTH-PBE-q6 \n"
        "    &END KIND \n";
    }
    else 
    {
        FLAW ("Unknown DFT");
    }

    return text;
}

Text Panel_CP2K::
data ()
{
    Text text;

    for (int i=0;  i<model_->atom_count();  ++i)
    {
        Atom const & atom = model_->atom(i);

        text += " ";
        text += atom.element().c_str();
        text += "    ";
        text += Text(atom.x(), 15);
        text += " ";
        text += Text(atom.y(), 15);
        text += " ";
        text += Text(atom.z(), 15);
        text += "   ! ";
        text += i + 1;
        text += "\n";
    }

    return text;
}

bool Panel_CP2K::
not_semiempirical ()
{
    //Text basis = comboBox_Basis->currentText();
    //Text panel = tabWidget_Basis->currentWidget()->objectName();

    //if (panel == "Assembly" &&
    //    (basis == "MNDO" || basis == "AM1" || basis == "PM3" || basis == "RM1"))
    //    return false;

    return true;
}

}//MM
